<?php

/**
 * @file
 * Filters and Input Formats.
 */

/**
 * Install a new input format.
 *
 * A sample array for a new input format to pass.
 * <code>
 * $format = array(
 *   'name' => 'Recipe Description',
 *   'allowed-html' => '<a> <em> <strong>',
 * );
 * $format['roles'] = array(
 *   'authenticated user',
 *   'anonymous user',
 * );
 * $format['filters'] = array(
 *   // 'module' => array(
 *   //   'Filter Name' => 'weight',
 *   // ),
 *   'filter' => array(
 *     'HTML filter'          => 0,
 *     'Line break converter' => 1,
 *     'URL filter'           => 2,
 *     'HTML corrector'       => 3,
 *   ),
 * );
 * $format['wyswig'] = array(
 *   'editor' => 'tinymce',
 *   'settings' => array(), // An array of settings, exported from the database.
 * );
 * </code>
 *
 * @param $ret
 *   Return array in update_sql format.
 * @param $format
 *   The format array, as described above.
 * @return
 *   The format_id of the created input format
 */
function update_api_install_input_format(&$ret, $format) {
  $format += _update_api_input_format_defaults();
  $success = TRUE;

  // Can't do anything without a name!
  if (!isset($format['name'])) {
    $message = 'No name was found for this input format.';
    $success = FALSE;
  }

  if ($success) {
    $t_args = array(
      '@name' => $format['name'],
    );
    // Create the format.
    $success = ($format_id = _update_api_insert_format($format['name']));
    $message = 'Creating input format with name @name.';
  }

  $ret[] = array(
    'success' => $success,
    'query' => t($message, $t_args),
  );

  // Only continue further processing if we have been successful thus far.
  if ($success) {
    if ($format['default']) {
      variable_set('filter_default_format', $format_id);
    }

    if (is_string($format['allowed-html'])) {
      variable_set('allowed_html_'. $format_id, (string) $format['allowed-html']);
    }

    if (isset($format['roles'])) {
      $roles = array();
      foreach ($format['roles'] as $name) {
        if ($rid = db_result(db_query("SELECT rid FROM {role} WHERE name = '%s'", trim($name)))) {
          $roles[] = $rid;
        }
      }
      if (!empty($roles)) {
        update_api_format_set_roles($ret, $roles, $format_id);
      }
    }

    if (isset($format['filters'])) {
      db_query("DELETE FROM {filters} WHERE format = %d", $format_id);
      foreach($format['filters'] as $module => $filters) {
        $t_args['@module'] = $module;
        $function = "{$module}_filter";
        if (function_exists($function) && is_array($filters)) {
          $filter_list = $function('list');
          $deltas = array_flip($filter_list);
          foreach($filters as $name => $weight) {
            if (isset($deltas[$name])) {
              $t_args['@filter-name'] = $name;
              $result = db_query("INSERT INTO {filters} (format, module, delta, weight) VALUES (%d, '%s', %d, %d)", $format_id, $module, $deltas[$name], $weight);
              $ret[] = array(
                'success' => $result !== FALSE,
                'query' => t('Adding @filter-name to the @name input format.', $t_args),
              );
            }
          }
        }
      }
    }

    if (isset($format['wysiwyg']) && module_exists('wysiwyg')) {
      $editor = $format['wysiwyg']['editor'];
      $settings = serialize($format['wysiwyg']['settings']);

      // Insert new profile data.
      db_query("UPDATE {wysiwyg} SET settings = '%s', editor = '%s' WHERE format = %d", $settings, $editor, $format_id);
      if (!db_affected_rows()) {
        db_query("INSERT INTO {wysiwyg} (format, editor, settings) VALUES (%d, '%s', '%s')", $format_id, $editor, $settings);
      }
    }
  }

  return $format_id;
}

/**
 * Set the roles that can be used with the filter.
 *
 * @param $roles
 *   An array of role IDs.
 * @param $format_id
 *   An integer of the format ID.
 */
function update_api_format_set_roles(&$ret, $roles, $format_id) {
  $roles = implode(',',$roles);
  // Drupal core depends on the list of roles beginning and ending with commas.
  if (!empty($roles)) {
    $roles = ','. $roles .',';
  }
  $result = db_query("UPDATE {filter_formats} SET roles = '%s' WHERE format = %d", $roles, $format_id);
}

/**
 * Add a new input format .
 *
 * @param $name
 *   The human-readable name of the new format.
 * @param $cache
 *   If this format is cacheable.
 * @return
 *   The new format ID.
 */
function _update_api_insert_format($name, $cache = 1) {
  db_query("INSERT INTO {filter_formats} (name, roles, cache) VALUES ('%s', '', %d)", $name, $cache);
  return db_last_insert_id('filter_formats', 'format');
}

/**
 * Helper function to return an array of format defaults.
 */
function _update_api_input_format_defaults() {
  return array(
    'default' => FALSE,
    'allowed-html' => NULL,
    'roles' => NULL,
    'filters' => NULL,
    'wysiwyg' => NULL,
  );
}